<?php

/**
 * @file
 * Rules integration for amoCRM.
 */

/**
 * Implements hook_rules_action_info().
 * @ingroup rules
 */
function amocrm_rules_action_info() {
  try {
    $client = amocrm_api_get_default_client();
    $client_info = amocrm_api_account_load($client);
    $entities = array('contacts', 'leads', 'companies');

    if (!empty($client_info)) {
      foreach ($entities as $entity_name) {
        $entities_info[$entity_name] = array();

        // Get all available custom fields for the current entity.
        if (!empty($client_info['custom_fields'][$entity_name])) {
          foreach ($client_info['custom_fields'][$entity_name] as $entity_custom_field) {
            if (empty($entity_custom_field['enums'])) {
              $entities_info[$entity_name][$entity_custom_field['id']] = array(
                'label' => $entity_custom_field['name'],
                'type' => 'text',
                'optional' => TRUE,
              );
            }
            // To get all add-on fields with the key "enum".
            else {
              foreach ($entity_custom_field['enums'] as $enum_key => $enum_name) {
                $entities_info[$entity_name][$entity_custom_field['id'] . '_' . $enum_key] = array(
                  'label' => t('!field_name - (!enum_name)', array(
                    '!enum_name' => $enum_name,
                    '!field_name' => $entity_custom_field['name'])),
                  'type' => 'text',
                  'optional' => TRUE,
                );
              }
            }
          }
        }
      }
    }

    return array(
      'contacts' => array(
        'label' => t('Create contact in amoCRM'),
        'group' => t('amoCRM'),
        'named parameter' => TRUE,
        'base' => '_amocrm_entity_create',
        'parameter' => array(
          'responsible_user_id' => array(
            'label' => t('Responsible user'),
            'type' => 'integer',
            'optional' => TRUE,
            'options list' => '_amocrm_user_list',
          ),
          'tags' => array(
            'label' => t('Tags'),
            'description' => t('Use a comma to separate the tags.'),
            'type' => 'text',
            'optional' => TRUE,
          ),
          'name' => array(
            'label' => t('Name'),
            'type' => 'text',
          ),
        ) + (!empty($entities_info['contacts']) ? $entities_info['contacts'] : array()),
      ),
      'companies' => array(
        'label' => t('Create company in amoCRM'),
        'group' => t('amoCRM'),
        'named parameter' => TRUE,
        'base' => '_amocrm_entity_create',
        'parameter' => array(
          'responsible_user_id' => array(
            'label' => t('Responsible user'),
            'type' => 'integer',
            'optional' => TRUE,
            'options list' => '_amocrm_user_list',
          ),
          'tags' => array(
            'label' => t('Tags'),
            'description' => t('Use a comma to separate the tags.'),
            'type' => 'text',
            'optional' => TRUE,
          ),
          'name' => array(
            'label' => t('Name'),
            'type' => 'text',
          ),
        ) + (!empty($entities_info['companies']) ? $entities_info['companies'] : array()),
      ),
      'leads' => array(
        'label' => t('Create lead in amoCRM'),
        'group' => t('amoCRM'),
        'named parameter' => TRUE,
        'base' => '_amocrm_entity_create',
        'parameter' => array(
          'responsible_user_id' => array(
            'label' => t('Responsible user'),
            'type' => 'integer',
            'optional' => TRUE,
            'options list' => '_amocrm_user_list',
          ),
          'tags' => array(
            'label' => t('Tags'),
            'description' => t('Use a comma to separate the tags.'),
            'type' => 'text',
            'optional' => TRUE,
          ),
          'name' => array(
            'label' => t('Name'),
            'type' => 'text',
          ),
          'price' => array(
            'label' => t('Price'),
            'type' => 'integer',
            'optional' => TRUE,
          ),
          'status_id' => array(
            'label' => t('Leads statuses'),
            'type' => 'integer',
            'optional' => TRUE,
            'options list' => '_amocrm_lead_statuses_list',
          ),
        ) + (!empty($entities_info['leads']) ?  $entities_info['leads'] : array()),
      ),
    );
  }
  catch (Exception $e) {
    watchdog('amoCRM', 'Can not declare Rules Action, without configured connection to amoCRM', array(), WATCHDOG_ERROR);
    return array();
  }
}

/**
 * Implements hook_rules_event_info().
 */
function amocrm_rules_event_info() {
  $items = array();
  $types = array(AMOCRM_TYPE_LEAD, AMOCRM_TYPE_CONTACT, AMOCRM_TYPE_COMPANY);

  $vars[AMOCRM_TYPE_LEAD] = array(
    'amocrm' => array(
      'type' => 'amocrm_' . AMOCRM_TYPE_LEAD,
      'label' => t('Input leads data from amoCRM webhook.'),
      'description' => t('When lead is deleted, there is no available tokens, except lead ID.'),
    ),
  );
  $vars[AMOCRM_TYPE_CONTACT] = array(
    'amocrm' => array(
      'type' => 'amocrm_' . AMOCRM_TYPE_CONTACT,
      'label' => t('Input contacts data from amoCRM webhook.'),
      'description' => t('When contact is deleted, there is no available tokens, except contact ID.'),
    ),
  );
  $vars[AMOCRM_TYPE_COMPANY] = array(
    'amocrm' => array(
      'type' => 'amocrm_' . AMOCRM_TYPE_COMPANY,
      'label' => t('Input companies data from amoCRM webhook.'),
      'description' => t('When company is deleted, there is no available tokens, except company ID.'),
    ),
  );

  foreach ($types as $type) {
    $items['amocrm_' . $type . '_' . AMOCRM_CRUD_OPERATIONS_CREATE] = array(
      'label' => t('After saving new !type', array('!type' => $type)),
      'group' => t('amoCRM'),
      'variables' => $vars[$type],
    );
    $items['amocrm_' . $type . '_' . AMOCRM_CRUD_OPERATIONS_UPDATE] = array(
      'label' => t('After updating existing !type', array('!type' => $type)),
      'group' => t('amoCRM'),
      'variables' => $vars[$type],
    );
    $items['amocrm_' . $type . '_' . AMOCRM_CRUD_OPERATIONS_DELETE] = array(
      'label' => t('After deleting !type', array('!type' => $type)),
      'group' => t('amoCRM'),
      'variables' => $vars[$type],
    );

    if ($type == AMOCRM_TYPE_LEAD) {
      $items['amocrm_' . $type . '_' . AMOCRM_CRUD_OPERATIONS_STATUS] = array(
        'label' => t('After changing status of existing !type', array('!type' => $type)),
        'group' => t('amoCRM'),
        'variables' => $vars[$type],
      );
      $items['amocrm_' . $type . '_' . AMOCRM_CRUD_OPERATIONS_RESTORE] = array(
        'label' => t('After restoring deleted !type', array('!type' => $type)),
        'group' => t('amoCRM'),
        'variables' => $vars[$type],
      );
    }
  }

  return $items;
}

/**
 * Rules action.
 */
function _amocrm_entity_create($args, $element) {
  try {
    $client = amocrm_api_get_default_client();
    $client_info = amocrm_api_account_load($client);
    $entity_type = $element->getElementName();

    $info = array(
      'name' => !empty($args['name']) ? $args['name'] : '',
      'tags' => !empty($args['tags']) ? $args['tags'] : '',
      'date_create' => REQUEST_TIME,
      'last_modified' => REQUEST_TIME,
      'responsible_user_id' => !empty($args['responsible_user_id']) ? $args['responsible_user_id'] : '',
    );

    if (!empty($client_info)) {
      $info['custom_fields'] = _amocrm_prepare_info($entity_type, $client_info, $args);

      switch ($entity_type) {
        case 'contacts':
          amocrm_api_contacts_add($client, array($info));
          break;

        case 'companies':
          amocrm_api_companies_add($client, array($info));
          break;

        case 'leads':
          $info += array(
            'price' => !empty($args['price']) ? $args['price'] : 0,
            'status_id' => !empty($args['status_id']) ? $args['status_id'] : 0,
          );
          amocrm_api_leads_add($client, array($info));
          break;
      }
    }
    else {
      watchdog('amocrm_contact', 'Not filled with data to connect to amoCRM', array(), WATCHDOG_ERROR);
    }
  }
  catch (Exception $e) {
    $message = 'amoCRM Error (@code): @message';
    $args = array(
      '@message' => $e->getMessage(),
      '@code' => $e->getCode(),
    );
    watchdog('amoCRM', $message, $args, WATCHDOG_ERROR);
  }
}

/**
 * Prepare info for amoCRM request.
 */
function _amocrm_prepare_info($type, $client_info, $params) {
  $custom_fields = array();
  if (!empty($client_info['custom_fields'][$type])) {
    foreach ($client_info['custom_fields'][$type] as $entity_custom_field) {
      if (!empty($entity_custom_field['enums'])) {
        foreach ($entity_custom_field['enums'] as $enum_key => $enum_name) {
          $value_key = $entity_custom_field['id'] . '_' . $enum_key;
          if (!empty($params[$value_key])) {
            $custom_fields[] = array(
              'id' => $entity_custom_field['id'],
              'values' => array(
                array(
                  'value' => $params[$value_key],
                  'enum' => $enum_key,
                ),
              ),
            );
          }
        }
      }
      else {
        if (!empty($params[$entity_custom_field['id']])) {
          $custom_fields[] = array(
            'id' => $entity_custom_field['id'],
            'values' => array(
              array(
                'value' => $params[$entity_custom_field['id']],
              ),
            ),
          );
        }
      }
    }
  }

  return $custom_fields;
}

/**
 * Get leads statuses.
 */
function _amocrm_lead_statuses_list() {
  $client = amocrm_api_get_default_client();
  $client_info = amocrm_api_account_load($client);
  $leads_statuses = array();

  if (!empty($client_info['leads_statuses'])) {
    foreach ($client_info['leads_statuses'] as $status) {
      $leads_statuses[$status['id']] = $status['name'];
    }
  }

  return $leads_statuses;
}

/**
 * Get leads statuses.
 */
function _amocrm_user_list() {
  $client = amocrm_api_get_default_client();
  $client_info = amocrm_api_account_load($client);
  $users = array();

  if (!empty($client_info['users'])) {
    foreach ($client_info['users'] as $user) {
      $users[$user['id']] = $user['name'] . ' ' . $user['last_name'];
    }
  }

  return $users;
}
